@@ -23,7 +23,7 @@
 #' @param region.table \code{data.frame} with two columns:
 #'        \tabular{ll}{ \code{Region.Label} \tab label for the region\cr
 #'                     \code{Area} \tab area of the region\cr}
-#'        \code{region.table} has one row for each stratum. If there is no stratification then \code{region.table} has one entry with \code{Area} corresponding to the total survey area.
+#'        \code{region.table} has one row for each stratum. If there is no stratification then \code{region.table} has one entry with \code{Area} corresponding to the total survey area. If \code{Area} is omitted density estimates only are produced.
 #' @param sample.table \code{data.frame} mapping the regions to the samples (i.e. transects). There are three columns:
 #'        \tabular{ll}{\code{Sample.Label} \tab label for the sample\cr
 #'                     \code{Region.Label} \tab label for the region that the
@@ -36,6 +36,7 @@
 #'                          sample belongs to.\cr
 #'                     \code{Sample.Label} \tab label for the sample\cr}
 #' @param convert.units conversion between units for abundance estimation, see "Units", below. (Defaults to 1, implying all of the units are "correct" already.)
+#' @param er.var encounter rate variance estimator to use when abundance estimates are required. Defaults to "R2" for line transects and "P3" for point transects. See \code{\link{dht2}} for more information and if more complex options are required.
 #' @param method optimization method to use (any method usable by \code{\link{optim}} or \pkg{optimx}). Defaults to \code{"nlminb"}.
 #' @param debug.level print debugging output. \code{0}=none, \code{1-3} increasing levels of debugging output.
 #' @param quiet suppress non-essential messages (useful for bootstraps etc). Default value \code{FALSE}.
@@ -49,6 +50,8 @@
 #' @section Details:
 #' If abundance estimates are required then the \code{data.frame}s \code{region.table} and \code{sample.table} must be supplied. If \code{data} does not contain the columns \code{Region.Label} and \code{Sample.Label} then the \code{data.frame} \code{obs.table} must also be supplied. Note that stratification only applies to abundance estimates and not at the detection function level.
 #'
+#' For more advanced abundance/density estimation please see the \code{\link{dht}} and \code{\link{dht2}} functions.
+#'
 #' Examples of distance sampling analyses are available at \url{http://examples.distancesampling.org/}.
 #'
 #' @section Clusters/groups:
@@ -59,7 +62,7 @@
 #'
 #' When specified as a percentage, the largest \code{right} and smallest \code{left} percent distances are discarded. Percentages cannot be supplied when using binned data.
 #'
-#' For left truncation, there are two options: (1) fit a detection function to the truncated data as is (this is what happens when you set \code{left}). This does not assume that g(x)=1 at the truncation point. (2) manually remove data with distances less than the left truncation distance -- effectively move the centreline out to be the truncation distance (this needs to be done before calling \code{ds}). This then assumes that detection is certain at the left truncation distance. The former strategy has a weaker assumption, but will give higher variance as the detection function close to the line has no data to tell it where to fit -- it will be relying on the data from after the left truncation point and the assumed shape of the detection function. The latter is most appropriate in the case of aerial surveys, where some area under the plane is not visible to the observers, but their probability of detection is certain at the smallest distance.
+#' For left truncation, there are two options: (1) fit a detection function to the truncated data as is (this is what happens when you set \code{left}). This does not assume that g(x)=1 at the truncation point. (2) manually remove data with distances less than the left truncation distance -- effectively move the centre line out to be the truncation distance (this needs to be done before calling \code{ds}). This then assumes that detection is certain at the left truncation distance. The former strategy has a weaker assumption, but will give higher variance as the detection function close to the line has no data to tell it where to fit -- it will be relying on the data from after the left truncation point and the assumed shape of the detection function. The latter is most appropriate in the case of aerial surveys, where some area under the plane is not visible to the observers, but their probability of detection is certain at the smallest distance.
 #'
 #' @section Binning:
 #' Note that binning is performed such that bin 1 is all distances greater or equal to cutpoint 1 (>=0 or left truncation distance) and less than cutpoint 2. Bin 2 is then distances greater or equal to cutpoint 2 and less than cutpoint 3 and so on.
@@ -82,10 +85,10 @@
 #'  easiest if the units of \code{Area} are the square of the units of
 #'  \code{Effort} and then it is only necessary to convert the units of
 #'  \code{distance} to the units of \code{Effort}. For example, if \code{Effort}
-#'   was entered in kilometers and \code{Area} in square kilometers and
-#'  \code{distance} in meters then using \code{convert.units=0.001} would
-#'  convert meters to kilometers, density would be expressed in square
-#'  kilometers which would then be consistent with units for \code{Area}.
+#'   was entered in kilometres and \code{Area} in square kilometres and
+#'  \code{distance} in metres then using \code{convert.units=0.001} would
+#'  convert metres to kilometres, density would be expressed in square
+#'  kilometres which would then be consistent with units for \code{Area}.
 #'  However, they can all be in different units as long as the appropriate
 #'  composite value for \code{convert.units} is chosen.  Abundance for a survey
 #'  region can be expressed as: \code{A*N/a} where \code{A} is \code{Area} for
@@ -94,16 +97,18 @@
 #'  \code{Effort * distance}.  The sampled region \code{a} is multiplied by
 #'   \code{convert.units}, so it should be chosen such that the result is in
 #'  the same units as \code{Area}.  For example, if \code{Effort} was entered
-#'  in kilometers, \code{Area} in hectares (100m x 100m) and \code{distance}
-#'  in meters, then using \code{convert.units=10} will convert \code{a} to
-#'  units of hectares (100 to convert meters to 100 meters for distance and
+#'  in kilometres, \code{Area} in hectares (100m x 100m) and \code{distance}
+#'  in metres, then using \code{convert.units=10} will convert \code{a} to
+#'  units of hectares (100 to convert metres to 100 metres for distance and
 #'  .1 to convert km to 100m units).
 #'
 #' @section Data format:
 #' One can supply \code{data} only to simply fit a detection function. However, if abundance/density estimates are necessary further information is required. Either the \code{region.table}, \code{sample.table} and \code{obs.table} \code{data.frame}s can be supplied or all data can be supplied as a "flat file" in the \code{data} argument. In this format each row in data has additional information that would ordinarily be in the other tables. This usually means that there are additional columns named: \code{Sample.Label}, \code{Region.Label}, \code{Effort} and \code{Area} for each observation. See \code{\link{flatfile}} for an example.
+#' @section Density estimation:
+#' If column \code{Area} is omitted, a density estimate is generated but note that the degrees of freedom/standard errors/confidence intervals will not match density estimates made with the \code{Area} column present.
 #'
 #' @author David L. Miller
-#' @seealso \code{\link{flatfile}}
+#' @seealso \code{\link{flatfile}} \code{\link{AIC.ds}} \code{\link{ds.gof}} \code{\link{p_dist_table}} \code{\link{plot.ds}} \code{\link{add_df_covar_line}}
 #' @export
 #'
 #' @importFrom stats quantile as.formula
@@ -176,7 +181,8 @@ ds <- function(data, truncation=ifelse(is.null(cutpoints),
                                  "strict",
                                  "none"),
              region.table=NULL, sample.table=NULL, obs.table=NULL,
-             convert.units=1, method="nlminb", quiet=FALSE, debug.level=0,
+             convert.units=1, er.var="R2",
+             method="nlminb", quiet=FALSE, debug.level=0,
              initial.values=NULL, max.adjustments=5){
 
   # capture the call
@@ -254,6 +260,7 @@ ds <- function(data, truncation=ifelse(is.null(cutpoints),
       message("Columns \"distbegin\" and \"distend\" in data: performing a binned analysis...")
       binned <- TRUE
       breaks <- sort(unique(c(data$distend, data$distbegin)))
+      data$distance <- (data$distend + data$distbegin)/2
     }else{
       binned <- FALSE
       breaks <- NULL
@@ -598,15 +605,22 @@ ds <- function(data, truncation=ifelse(is.null(cutpoints),
     stop("No models could be fitted.")
   }
 
+  # check that hazard models have a reasonable scale parameter
+  if(key=="hr" && model$par[1] < sqrt(.Machine$double.eps)){
+    warning("Estimated hazard-rate scale parameter close to 0 (on log scale). Possible problem in data (e.g., spike near zero distance).")
+  }
+
   ## Now calculate abundance/density using dht()
   if(!is.null(region.table) & !is.null(sample.table)){
-
-    ervar <- "R2"
-    if(point){
-      ervar <- "P3"
-    }
     # if obs.table is not supplied, then data must have the Region.Label and
     # Sample.Label columns
+
+    # setup dht options
+    dht_options <- list(group         = dht.group,
+                        ervar         = er.var,
+                        convert.units = convert.units)
+
+    # if no obs.table
     if(is.null(obs.table)){
       if(all(c("Region.Label", "Sample.Label") %in% names(data))){
 
@@ -615,17 +629,13 @@ ds <- function(data, truncation=ifelse(is.null(cutpoints),
           dht.res <- NULL
         }else{
           dht.res <- dht(model, region.table, sample.table,
-                         options=list(#varflag=0,
-                                      group         = dht.group,
-                                      ervar         = ervar,
-                                      convert.units = convert.units), se=TRUE)
+                         options=dht_options, se=TRUE)
         }
       }else{
         message("No obs.table supplied nor does data have Region.Label and Sample.Label columns, only estimating detection function.\n")
         dht.res <- NULL
       }
     }else{
-
       # from ?dht:
       # For animals observed in tight clusters, that estimator gives the
       # abundance of groups (group=TRUE in options) and the abundance of
@@ -638,10 +648,7 @@ ds <- function(data, truncation=ifelse(is.null(cutpoints),
         dht.res <- NULL
       }else{
         dht.res <- dht(model, region.table, sample.table, obs.table,
-                       options=list(#varflag=0,group=TRUE,
-                                    group         = dht.group,
-                                    ervar         = ervar,
-                                    convert.units = convert.units), se=TRUE)
+                       options=dht_options, se=TRUE)
       }
     }
   }else{
